#!/usr/bin/env python
import os
import pygtk
pygtk.require('2.0')
import gtk
import gobject

from gmusicapi import Mobileclient
from music_player import MusicPlayer
from widget import (SourcePane, ArtistPane, AlbumPane, SongPane,
                    get_player_control_toolbar, MainWindow, Login)

api = Mobileclient()

gobject.threads_init()


class Player:

    def __init__(self):
        self.window = MainWindow(gtk.WINDOW_TOPLEVEL)
        path = os.path.expanduser("~/.config/google_music_player")
        if not os.path.exists(path):
            message = Login(parent=self.window)
            message.run()
        path = os.path.expanduser("~/.config/google_music_player")
        while True:
            with open(path, "r") as fd:
                login, password = fd.readlines()
                logged = api.login(
                    login.strip(),
                    password.strip(),
                    Mobileclient.FROM_MAC_ADDRESS)
                if logged:
                    break
                else:
                    message = Login(parent=self.window)
                    message.run()

        self.t = None # The music thread
        # Create a new window

        self.window.connect("delete_event", self.delete_event)

        # create the liststores (artist, album, songs)
        self.make_stores()
        self.refresh()
        # create the TreeView using liststore

        self.source_pane = SourcePane(self.source_store)
        self.source_scrolled_window = gtk.ScrolledWindow()

        self.source_scrolled_window.add(self.source_pane)

        self.artist_pane = ArtistPane(self.artist_store)
        self.artist_scrolled_window = gtk.ScrolledWindow()

        self.artist_scrolled_window.add(self.artist_pane)

        self.album_pane = AlbumPane(self.album_store)
        self.album_scrolled_window = gtk.ScrolledWindow()
        self.album_scrolled_window.add(self.album_pane)

        self.treeview = SongPane(self.liststore)
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.add(self.treeview)

        self.main_hbox = gtk.HBox()
        self.hbox = gtk.HBox()
        self.hbox.add(self.source_scrolled_window)
        self.hbox.add(self.artist_scrolled_window)
        self.hbox.add(self.album_scrolled_window)

        self.menu_hbox = gtk.HBox()
        self.song_label = gtk.Label("")

        self.completion_store = gtk.ListStore(str, str, str)
        completion = gtk.EntryCompletion()
        completion.set_model(self.completion_store)
        completion.set_text_column(0)
        self.search = gtk.Entry()
        self.search.set_completion(completion)
        self.search.connect("key-press-event", self.autocomplete)
        self.search.connect("activate", self.get_selection)
        self.album_pic = gtk.Image()
        self.menu_hbox.pack_start(self.album_pic)
        self.menu_hbox.pack_start(get_player_control_toolbar(self))
        self.menu_hbox.pack_start(self.song_label)
        self.menu_hbox.pack_start(self.search)

        self.vbox = gtk.VBox()
        self.vbox.pack_start(self.menu_hbox, expand=False)
        self.vbox.add(self.hbox)

        self.vbox.add(self.scrolled_window)

        self.window.add(self.vbox)

        self.window.show_all()

        self.source_pane.connect("row-activated", self.expand)
        self.treeview.connect("row-activated", self.on_clicked)
        self.album_pane.get_selection().connect("changed", self.filter_album)
        self.artist_pane.get_selection().connect("changed", self.filter_artist)

    def get_selection(self, widget, *args):
        text = widget.get_text()
        obj_type = None
        tracks = None
        for row in self.completion_store:
            if row[0] == text:
                name, obj_id, obj_type = row[0], row[1], row[2]
                break
        if obj_type == "artist":
            tracks = api.get_artist_info(
                obj_id, max_top_tracks=20)['topTracks']
        elif obj_type == "album":
            tracks = api.get_album_info(
                obj_id
            )['tracks']
        if tracks:
            for entry in tracks:
                entry["id"] = entry["storeId"]
            self.refresh(tracks)

    def autocomplete(self, widget, key):
        string = u"{}{}".format(self.search.get_text(), key.string)
        results = api.search_all_access(string, max_results=10)
        self.completion_store.clear()
        if results.get('artist_hits'):
            for artist in results.get('artist_hits'):

                self.completion_store.append([artist["artist"]["name"],
                                             artist["artist"]["artistId"],
                                              "artist"])
        if results.get('album_hits'):
            for album in results.get('album_hits'):

                self.completion_store.append([album["album"]["name"],
                                              album["album"]["albumId"],
                                              "album"])
        # print results.get('playlist_hits')
        # print results.get('song_hits')[0]

    # close the window and quit
    def delete_event(self, widget, event, data=None):
        if self.t:
            self.t.join()
        gtk.main_quit()
        return False

    def make_stores(self):
        self.liststore = gtk.ListStore(str, str, str, str, str)
        self.album_store = gtk.ListStore(str)
        self.artist_store = gtk.ListStore(str)
        self.source_store = gtk.TreeStore(str)
        for elem in ["library", "playlists", "radios"]:
            self.source_store.append(None, [elem])

    def refresh(self, songs=None):
        if songs is None:
            self.songs = api.get_all_songs()
        else:
            self.songs = songs
        for store in [self.liststore, self.album_store, self.artist_store]:
            store.clear()
        for album in set([song['album'] for song in self.songs]):
            self.album_store.append([album])

        for artist in set([song['artist'] for song in self.songs]):
            self.artist_store.append([artist])

        for song in self.songs:
            self.liststore.append(
                [song["artist"],
                 song['album'],
                 song["title"],
                 song['id'],
                 song['albumArtRef'][0]['url']])


    def filter_album(self, selection):
        model, index = selection.get_selected_rows()
        index = index[0][0]
        album = self.album_store[index][0]
        self.liststore.clear()
        for song in self.songs:
            if song['album'] == album:
                self.liststore.append(
                    [song["artist"],
                     song['album'],
                     song["title"],
                     song['id'],
                     song['albumArtRef'][0]['url']])

    def filter_artist(self, selection):
        model, index = selection.get_selected_rows()
        index = index[0][0]
        artist = self.artist_store[index][0]
        self.liststore.clear()
        album_songs = []
        for song in self.songs:
            if song['artist'] == artist:
                album_songs.append(song['album'])
                self.liststore.append(
                    [song["artist"],
                     song['album'],
                     song["title"],
                     song['id'],
                     song['albumArtRef'][0]['url']])

        self.album_store.clear()
        for album in set(album_songs):
            self.album_store.append([album])

    def on_clicked(self, widget, index, item):
        index = index[0]
        self.play(index= index)

    def play(self, index=None):
        if self.t:
            self.t.join()
        selection = self.treeview.get_selection()
        try:
            index = selection.get_selected_rows()[1][0][0]
        except IndexError:
            index = 0
        self.t = MusicPlayer(api, self, index, self.treeview)
        self.t.start()

    def pause(self):
        if self.t:
            self.t.p.pause()

    def stop(self):
        if self.t:
            self.t.p.stop()
            self.t.join()

    def expand(self, widget, index, item):
        index = index
        it = self.source_store.get_iter(index)
        menu = self.source_store.get(it, 0)
        if len(index) == 1:
            if menu[0] == "playlists":
                self.playlists = api.get_all_user_playlist_contents()
                for playlist in self.playlists:
                    self.source_store.append(
                        self.source_store.get_iter(1),
                        [playlist["name"]])
            if menu[0]== "radios":
                self.radios = api.get_all_stations()
                for radio in self.radios:
                    self.source_store.append(
                        self.source_store.get_iter(2),
                        [radio["name"]])
        else:
            source_type = self.source_store.get_iter(index[0])
            if self.source_store.get(source_type, 0)[0] == "playlists":
                playlist = (
                    item for item in self.playlists
                    if item["name"] == menu[0]
                ).next()
                for entry in playlist['tracks']:
                    try:
                        entry["track"]["id"] = entry["trackId"]
                    except KeyError:
                        pass
                tracks = []
                for entry in playlist['tracks']:
                    if entry.get("track"):
                        tracks.append(entry['track'])
                self.refresh(tracks)
            elif self.source_store.get(source_type, 0)[0] == "radios":
                radio = (
                    item for item in self.radios
                    if item["name"] == menu[0]
                ).next()
                tracks = []
                for track in api.get_station_tracks(radio['id']):
                    track['id'] = track['nid']
                    tracks.append(track)
                self.refresh(tracks)
def main():
    gtk.main()

if __name__ == "__main__":

    player = Player()
    main()



import gtk
import os
from gettext import gettext as _

def get_player_control_toolbar(win):
    """Return a player control toolbar
    """
    tb = gtk.Toolbar()
    tb.set_style(gtk.TOOLBAR_ICONS)
    for text, tooltip, stock, callback in (
            (_("Play"), _("Play"), gtk.STOCK_MEDIA_PLAY, lambda b: win.play()),
            (_("Pause"), _("Pause"), gtk.STOCK_MEDIA_PAUSE, lambda b: win.pause()),
            (_("Stop"), _("Stop"), gtk.STOCK_MEDIA_STOP, lambda b: win.stop()),
            (_("Refresh"),_("Refresh"), gtk.STOCK_REFRESH, lambda b: win.refresh())
            ):
        b=gtk.ToolButton(stock)
        b.set_tooltip_text(tooltip)
        b.connect("clicked", callback)
        tb.insert(b, -1)
        tb.show_all()
    return tb


class SourcePane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(SourcePane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Source')
        self.set_search_column(0)
        self.append_column(self.tvcolumn)
        self.cell = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, True)
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.tvcolumn.set_clickable(True)
        self.tvcolumn.set_sort_column_id(0)
        self.set_reorderable(False)


class ArtistPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(ArtistPane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Artist')
        self.set_search_column(0)
        self.append_column(self.tvcolumn)
        self.cell = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, True)
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.tvcolumn.set_clickable(True)
        self.tvcolumn.set_sort_column_id(0)
        self.set_reorderable(True)



class AlbumPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(AlbumPane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Album')
        self.set_search_column(0)
        self.tvcolumn.set_clickable(True)
        self.append_column(self.tvcolumn)
        self.cell = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, True)
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.set_reorderable(True)
        self.tvcolumn.set_sort_column_id(0)

class SongPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(SongPane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Title')
        self.tvcolumn1 = gtk.TreeViewColumn('Album')
        self.tvcolumn2 = gtk.TreeViewColumn('Artist')
        self.set_search_column(0)

        self.tvcolumn.set_sort_column_id(0)
        self.tvcolumn1.set_sort_column_id(0)
        self.tvcolumn.set_clickable(True)

        self.append_column(self.tvcolumn)
        self.append_column(self.tvcolumn1)
        self.append_column(self.tvcolumn2)
        self.cell = gtk.CellRendererText()
        self.cell1 = gtk.CellRendererText()
        self.cell2 = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, True)
        self.tvcolumn.set_attributes(self.cell, text=2)
        self.tvcolumn1.pack_start(self.cell1, True)
        self.tvcolumn1.set_attributes(self.cell1, text=1)
        self.tvcolumn2.pack_start(self.cell2, True)
        self.tvcolumn2.set_attributes(self.cell2, text=0)
        self.set_reorderable(True)


class LibraryPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(LibraryPane, self).__init__(*args, **kwargs)



class MainWindow(gtk.Window):

    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.set_title("Google Music Player")


class Login(gtk.Dialog):

    def __init__(self, *args, **kwargs):
        super(Login, self).__init__(*args, **kwargs)
        self.label = gtk.Label("set your mail and password in order to connect")
        self.mail = gtk.Entry()
        self.password = gtk.Entry()
        self.password.set_visibility(False)
        self.message = gtk.Dialog()
        self.button = gtk.Button(stock=gtk.STOCK_APPLY)
        self.button.connect("clicked", self.clicked)
        self.vbox.pack_start(self.label)
        self.vbox.pack_start(self.mail)
        self.vbox.pack_start(self.password)
        self.action_area.pack_start(self.button)
        self.show_all()

    def clicked(self, widget):
        user_email = self.mail.get_text()
        user_password = self.password.get_text()
        path = os.path.expanduser("~/.config/google_music_player")
        with open(path, "w") as fd:
            fd.write("{}\n".format(user_email))
            fd.write("{}\n".format(user_password))
        self.destroy()



import threading
import time
import vlc
import gtk
import urllib2
from gmusicapi.utils import utils
from uuid import getnode as getmac

mac_int = getmac()
if (mac_int >> 40) % 2:
    raise OSError("a valid MAC could not be determined."
                  " Provide an android_id (and be"
                  " sure to provide the same one on future runs).")

android_id = utils.create_mac_string(mac_int)
android_id = android_id.replace(':', '')

class MusicPlayer(threading.Thread):

    def __init__(self, api, win, index, tree):
        super(MusicPlayer, self).__init__()
        self.win = win
        self.store = win.liststore
        self.index = index
        self.tree = tree
        self.stoprequest = threading.Event()
        self.api = api

    def join(self, timeout=None):
        self.stoprequest.set()
        self.p.stop()
        super(MusicPlayer, self).join(timeout)

    def play(self):

        try:
            artist = self.store[self.index][0]
            album = self.store[self.index][1]
            song = self.store[self.index][2]
            text = "{}-{}-{}".format(
                    song,
                    album,
                    artist)
            if len(text) > 50:
                text = text[:50] + '...'
            self.win.song_label.set_text(
                text
                )
            song = self.api.get_stream_url(
                self.store[self.index][3],
                device_id=android_id)
            self.p=vlc.MediaPlayer(song)
            self.p.play()
            response=urllib2.urlopen(self.store[self.index][4])
            loader=gtk.gdk.PixbufLoader()
            loader.set_size(50,50)
            loader.write(response.read())
            loader.close()
            self.win.album_pic.set_from_pixbuf(loader.get_pixbuf())
            self.watch(self.index)
        except IndexError:
            return


    def watch(self, index):
        while True:
            state = self.p.get_state()

            if state == vlc.State.Ended:
                self.api.increment_song_playcount(self.store[self.index][-1])
                self.index += 1
                self.play()
            elif state == vlc.State.Stopped:
                break
            else:
                time.sleep(1)

    def run(self):
        self.play()
